/*
  Copyright (c) 2013-2015, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "parse.hpp"

#include <cstdio>
#include <cstring>


namespace solar {

struct location_t {
public:
	int first_line;
	int first_column;
	int last_line;
	int last_column;

	location_t() : first_line(1), first_column(0), last_line(1), last_column(0) {}
};

class lex_t {
private:
	location_t loc;

	std::FILE *file;

	bool needspace;
	bool dumb_mode;

	size_t start;
	size_t end;
	size_t tok_len;
	char buffer[1024];


	bool advance();
	bool next(bool move);
	void consume(bool needspace);

	int io_error(parse_token_value_t *value);
	int syntax_error(parse_token_value_t *value);
	int consume_comment(parse_token_value_t *value);
	int unterminated_block(parse_token_value_t *value);
	int unterminated_string(parse_token_value_t *value);

	int lex_string(parse_token_value_t *value);
	int lex_block(parse_token_value_t *value, int token, char open, char close);
	int lex_symbol(parse_token_value_t *value);

	char current() {
		return buffer[start + tok_len];
	}

	std::string * get_token() {
		return new std::string(buffer+start, tok_len);
	}


public:
	lex_t(FILE *file0) : file(file0), needspace(false), dumb_mode(false), start(0), end(0), tok_len(0) {
		advance();
	}

	~lex_t() {
		fclose(file);
	}

	int lex(parse_token_value_t *value);

	const location_t & get_location() const {
		return loc;
	}
};

}
